//**************************************************************************
//
// RC tank_no PWM ATtiny2313.c
//
// Author:	Jeremy Greenwood
//
// Description:
//
//		Microcontroller C code for the Bluetooth remote control tank
//
//**************************************************************************

#define F_CPU 8000000UL											// 8 MHz

#define USART_BAUDRATE 9600										// Define baud rate
#define BAUD_PRESCALE (((F_CPU / (USART_BAUDRATE * 8UL))) - 1)	// Double speed mode calculation for UBRR

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

// Motor pin Defines

#define MA1			PB0
#define MA2			PB1
#define MB1			PB2
#define MB2			PB3

// Basic Macros

#define off(bit) 	PORTB &= ~(1 << bit)						// Set 'bit' in PORTD low
#define on(bit) 	PORTB |= (1 << bit)							// Set 'bit' in PORTD high
#define flip(bit) 	PORTB ^= (1 << bit)							// Toggle 'bit' in PORTD
#define get(bit) 	(PINB & (1 << bit))							// Get value of 'bit' in PORTB (leave parenthesis)

// Motor Control Macros

#define	MOV_FWD() 	PORTB=(PORTB & 0b11110000) | 0b1010			// Calculation will not overwrite internal pullup resistors
#define	MOV_BCK() 	PORTB=(PORTB & 0b11110000) | 0b0101
#define TURN_R() 	PORTB=(PORTB & 0b11110000) | 0b1001
#define TURN_L() 	PORTB=(PORTB & 0b11110000) | 0b0110
#define STOP() 		PORTB=(PORTB & 0b11110000) | 0b0000

// Prototypes

void init_UART( void );
void Tx( unsigned char data );
unsigned char Rx( void );

volatile int	receive_flag = 0;

ISR( USART_RX_vect ) {

	receive_flag = 1;
}

int main (void) {

	char input;	

	DDRB |= (1 << MA1) | (1 << MA2)								// Define outputs for motor logic pins
		| (1 << MB1) | (1 << MB2);						

	init_UART();

	sei(); 														// Set Global Interrupt Enable

   	while(1) {
		
		if( receive_flag ) {

			input = Rx();										// Recieve byte

			switch( input ) {

				case '4':

					TURN_L();
					break;

				case '6':

					TURN_R();
					break;

				case '8':

					MOV_FWD();
					break;

				case '2':

					MOV_BCK();
					break;

				case '5':

					STOP();
					break;
			}

			receive_flag = 0;
		}		
	}
}

// ***************************************
// **** Initialize UART ******************	
// ***************************************

void init_UART( void ) {

// **** Enable Rx and Tx *****************

	UCSRB = (1 << RXEN) | (1 << TXEN)							// Enable Receive and Transmit
		| (1 << RXCIE);											// Enable Rx interrupt

// **** Set frame format: 8data, 1 stop***

	UCSRC |= (0 << USBS);										// Use 1 stop bit
																// UCSZ1:0 normally 1

// **** Set baud rate ********************

	UBRRL = BAUD_PRESCALE;									
	UBRRH = (BAUD_PRESCALE >> 8);

	UCSRA |= (1 << U2X);										// Set double baud rate
}

// ***************************************
// **** UART Transmit ********************	
// ***************************************

void Tx( unsigned char data ) {
	
// **** Wait for empty Tx buffer *********

	while( (UCSRA & (1 << UDRE)) == 0 ){};

// **** Put data in buffer, send data ****

	UDR = data;
}

// ***************************************
// **** UART Receive *********************	
// ***************************************

unsigned char Rx( void ) {

// **** Wait for data to be received *****

	while( (UCSRA & (1 << RXC)) == 0 ){};

// **** Return received data in buffer ***

	return UDR;
}
